//import java.util.*;
import java.awt.*;
import java.util.Arrays;

/**
最小木問題に対するクラスカル法のプログラム例
http://web1.kcg.edu/~t_ibaraki/shokodo/kruskal.c
を Java に移植した。
 */
@SuppressWarnings("serial")
public class Kruskal extends GraphApplication {
    public Kruskal(String title, String file) {
        super(title, file);
    }

    /** 現在の最小木を構成する枝の数 */
    private int k = 0;
    /** 最小木 */
    private Edge tree[];

    /** 枝 */
    private class Edge implements Comparable<Edge> {
        float d;            /* 枝長 */
        int end1, end2;     /* 両端点 */
        Edge(int end1, int end2, float d) {
            this.end1 = end1;
            this.end2 = end2;
            this.d = d;
        }
        public int compareTo(Edge other) {
            //return Double.compare(d, ((Edge)other).d);
            double diff = d - other.d;
            if (diff > 0) return 1;
            if (diff < 0) return -1;
            return 0;
        }
    }

    /** 森 */
    private class Set {
        int size;
        int root;
        int parent;
    }

    public void drawAllEdges( Graphics g ) {
        for (int i=0; i<k; i++) {
            drawEdge(tree[i].end1, tree[i].end2, g);
        }
    }

    /* 集合iとkの２つの木の併合 */
    private void mergeTree(int i, int j, Set s[]) {
        Set large, small;
        
        if (s[i].size <= s[j].size) { small = s[i]; large = s[j]; }
        else                        { small = s[j]; large = s[i]; }
        if (small.size == 0) return;
        s[small.root].parent = large.root;
        large.size += small.size;
        small.size = 0;
        small.root = -1;
    }

    /* jを要素とする集合iの出力と路の圧縮 */
    private int findTree(int index, Set s[]) {
        int k;
        for (k=index; k>=0; k=s[k].parent) /**/;
        final int ret = -k-1;

        for (k=index; k>=0; /*k=s[k].parent*/) {
            int temp = s[k].parent;
            if (temp>=0) s[k].parent = s[ret].root;
            k=temp;
        }
        return ret;
    }


    /* 節点数n, 枝数m, 枝集合Eをもつグラフの最小木を求める
       クラスカルのアルゴリズム; 結果を tree に入れる */
    public void kruskal(Edge e[], int n, int m) {
        k = 0;
        setSuspended( true );
        repaint();
        if ( !sleep() ) return;

        Set comp[] = new Set[n];     /* 計算に必要な集合族（森による表現） */
        for(int i=0; i<n; i++) {         /* compの初期化 */
            comp[i] = new Set();
            comp[i].size=1; comp[i].root=i; comp[i].parent=-i-1;
        }

        Arrays.sort(e);      /* 枝集合Eの整列 */
        
        /* クラスカルのアルゴリズムの反復 */
        for (int i=0; i<m; i++) {
            int s1 = findTree(e[i].end1, comp);  /* 枝e[i]の両端点の集合 */
            int s2 = findTree(e[i].end2, comp);
            if (s1 != s2) {             /* s1とs2の併合 */
                mergeTree(s1, s2, comp);
                tree[k] = e[i];
                drawEdge(tree[k].end1, tree[k].end2);
                if (++k == n-1) break;
                if ( !sleep() ) return;
            }
        }
        sleepForever();
    }

    public void run() {
        super.run();

        // 頂点座標を元に完全グラフを作る
        int m = n * (n-1) / 2;

        Edge[] edge = new Edge[m];
        tree = new Edge[n-1];
        int index = 0;
        for (int i=0; i<n; i++) {
            for (int j=0; j<i; j++) {
                edge[index++] = new Edge( i, j, d[i][j] );
            }
        }
        setReady(true);

        while (thread != null) kruskal( edge, n, m );
    }
}
